;; AquaX/theme.jl
;;    Copyright (C) 2000, 2002 Kevin D. Knerr, Sr. <ld_barthel@yahoo.com>
;;
;; See the file ChangeLog for details of changes to this theme.
;;
;; Requires sawfish v1.0.1 or later

;; AquaX is free software distributed under the terms
;; of the GNU General Public License, version 2, as published by the
;; Free Software Foundation. For details, see the file COPYING or visit
;; http://www.gnu.org/copyleft/gpl.html.

;; USAGE:
;;
;; To use AquaX, untar the distribution in your Sawfish thmese directory
;;
;; If you wish to use AquaX in its tarball form, make sure that the
;; filename stem is simply "AquaX", otherwise Sawfish will be unable to
;; access the theme.

;; 2002-Jan-06: v1.6.0 by Kevin D. Knerr, Sr. <ld_barthel@yahoo.com>
;;   "Three Weis Guys in Bethlehem, PA"

(require 'rep.data.tables)

(define theme-name 'AquaX)

;;;;
;;;; user-configurable options
;;;;

;; define the customization groups
(defgroup aquax "AquaX theme")

(defgroup aquax:buttons "Buttons & Cursors"
  :group aquax)

(defgroup aquax:colors "Fonts & Colors"
  :group aquax)

(defgroup aquax:icons "Icon Captions"
  :group aquax)


;; define the customization variables
(defcustom aquax:default-font 'default
  "Font selection source"
  :group (aquax aquax:colors)
  :type (choice (default "AquaX")
                (gtk "GTK theme")
                (sawfish "Sawfish default"))
  :after-set (lambda () (reframe-all)))

(defcustom aquax:font "-adobe-helvetica-bold-r-normal-*-*-140-*-*-p-*-*"
  "Title font."
  :group (aquax aquax:colors)
  :type font
  :after-set (lambda () (reframe-all)))

(define aquax:title-font
  (lambda ()
  (case aquax:default-font
    ((default)
      aquax:font)
    ((gtk)
      (require 'sawfish.wm.util.gtkrc)
      (gtkrc-call-after-changed reframe-all)
      gtkrc-font)
    ((sawfish)
      default-font)
  ))
)  

(defcustom aquax:focused-color "#000000000000"
  "Focused title text color."
  :group (aquax aquax:colors)
  :type color
  :after-set (lambda () (reframe-all)))

(defcustom aquax:inactive-color "#6b847fffbfff"
  "Inactive title text color."
  :group (aquax aquax:colors)
  :type color
  :after-set (lambda () (reframe-all)))

;; This option lets user choose whether the cursor should change when over
;;    a button. Mac OS X keeps the arrow cursor. A vaulue of "nil" mimics
;;    this behavior in AquaX
;; NOTE: Sawfish now keeps the default cursor for buttons, so in effect,
;;    this option allows the user to choose between current sawfish behavior
;;    and legacy sawmill behavior . . .

(defcustom aquax:button-cursors 'AquaX
  "Set of cursors displayed over buttons/titlebar"
  :group (aquax aquax:buttons)
  :type symbol
  :options (AquaX Sawfish Sawmill)
  :after-set (lambda () (reframe-all)))

(define (set-button-cursors class)
  (lambda ()
  (case class
    ((close-button)
      (if (eq aquax:button-cursors 'Sawmill)
        'dotbox
        'left_ptr))
    ((iconify-button)
      (if (eq aquax:button-cursors 'Sawmill)
        'sb_down_arrow
        'left_ptr))
    ((maximize-button)
      (if (eq aquax:button-cursors 'Sawmill)
        'sb_v_double_arrow
        'left_ptr))
    ((menu-button)
      (if (eq aquax:button-cursors 'Sawmill)
        'circle
        'left_ptr))
    ((shade-button)
      (if (eq aquax:button-cursors 'Sawmill)
        'top_tee
        'left_ptr))
    ((sticky-button)
      (if (eq aquax:button-cursors 'Sawmill)
        'diamond_cross
        'left_ptr))
    ((title)
      (if (eq aquax:button-cursors 'AquaX)
        'left_ptr
        'hand2))
  ))
)

(defcustom aquax:button-scheme 'default
  "Arrangement of buttons on titlebar"
  :group (aquax aquax:buttons)
  :type (choice (default "AquaX")
		(amiga "Amiga")
		(fvwm "FVWM2")
		(kde "KDE")
		(platinum "Mac OS Platinum")
		(windos "MS Windows")
		(next "NeXTSTEP")
		(twm "TWM")
		(windowmaker "WindowMaker")
		(custom "Custom"))
  :after-set (lambda () (reframe-all)))

;; Predefined button arrangements
(defvar aquax:button-schemes '(
  (default
    ((close-button iconify-button maximize-button) . (menu-button)))
  (amiga
    ((close-button) . (iconify-button maximize-button)))
  (fvwm
    ((menu-button sticky-button) . (shade-button iconify-button maximize-button)))
  (kde
    ((menu-button sticky-button) . (iconify-button maximize-button close-button)))
  (platinum
    ((close-button) . (maximize-button shade-button)))
  (windos
    ((menu-button) . (iconify-button maximize-button close-button))
    ((menu-button)))
  (next
    ((iconify-button) . (close-button))
    ((iconify-button) . (close-button)))
  (twm
    ((menu-button)))
  (custom
    ((menu-button sticky-button) . (shade-button iconify-button maximize-button close-button)))
))

(define button-table (make-weak-table eq-hash eq))


(defcustom aquax:faux-one_window 't
  "Menu button colored only when active
  (like Aqua's one-window button)?"
  :group (aquax aquax:buttons)
  :type boolean
  :after-set (lambda () (reframe-all)))

(defcustom aquax:shade-button-state 't
  "Shade button colored only when shaded
  (or unshaded by shade-hover)?"
  :group (aquax aquax:buttons)
  :type boolean
  :after-set (lambda () (reframe-all)))


;; These options affect the iconic frame and were tested w/ xiconic-0.4

(defcustom aquax:iconic-titles nil
  "Enable captions under icons?"
  :group (aquax aquax:icons)
  :type boolean
  :after-set (lambda () (reframe-all))
)

(defcustom aquax:iconic-font "-adobe-helvetica-medium-r-normal-*-*-100-*-*-p-*-*"
  "Icon font."
  :group (aquax aquax:icons)
  :type font
  :depends aquax:iconic-titles
  :after-set (lambda () (reframe-all))
)

(define aquax:icon-title-font
  (lambda ()
  (case aquax:default-font
    ((default)
      aquax:iconic-font)
    ((gtk)
      (require 'sawfish.wm.util.gtkrc)
      (gtkrc-call-after-changed reframe-all)
      gtkrc-font)
    ((sawfish)
      default-font)
  ))
)  

(defcustom aquax:iconic-focused-color "#f0a2f84fffff"
  "Focused icon text color."
  :group (aquax aquax:icons)
  :type color
  :depends aquax:iconic-titles
  :after-set (lambda () (reframe-all))
)

(defcustom aquax:iconic-inactive-color "#d1ead1ead1ea"
  "Inactive icon text color."
  :group (aquax aquax:icons)
  :type color
  :depends aquax:iconic-titles
  :after-set (lambda () (reframe-all))
)

;; We do have to set background colors if we want text to be displayed

(defcustom aquax:iconic-focused-bg-color "#00000000a147"
  "Focused icon background color."
  :group (aquax aquax:icons)
  :type color
  :depends aquax:iconic-titles
  :after-set (lambda () (reframe-all))
)

(defcustom aquax:iconic-inactive-bg-color "#00000000a147"
  "Inactive icon background color."
  :group (aquax aquax:icons)
  :type color
  :depends aquax:iconic-titles
  :after-set (lambda () (reframe-all))
)


;;;;
;;;; Define frame classes shade-button, sticky-button, & iconic-title
;;;;

;; Commented out. Current versions of sawfish include this class by default
;(def-frame-class shade-button ()
;  (bind-keys shade-button-keymap
;    "Button1-Off" 'toggle-window-shaded
;  )
;)  

(def-frame-class sticky-button ()
  (bind-keys sticky-button-keymap
    "Button1-Off" 'toggle-window-sticky
  )
)

(def-frame-class iconic-title ()
  (bind-keys iconic-title-keymap
    "Button3-Click1" 'raise-lower-window-and-transients
    "Button2-Click1" 'uniconify-window
    "Button1-Move" 'move-window-interactively
    "Button1-Off" 'popup-window-menu
  )
)


;;;; There are two methods for specifying state-dependent lists:
;;;;   a) a 4 item list consisting of normal, focused, highlighted, & clicked
;;;;      state values, in that order, with nil in place of unspecified valeues
;;;;      (although we can get away with just 2 items if they are normal &
;;;;      focused)
;;;;   b) an alist of cons cells (state . value), which is required if other
;;;;      states, such as inactive-highlighted or inactive-clicked are used)
;;;;      [Thanks to Ian  Bicking <ianb@colorstudy.com> for reminding me
;;;;      that the functions must be comma escaped to be evaluated correctly!]

(define font-colors
  (lambda () `(
    (inactive . ,aquax:inactive-color)
    (focused . ,aquax:focused-color)
  ))
)

(define iconic-font-colors
  (lambda () `(
    (inactive . ,aquax:iconic-inactive-color)
    (focused . ,aquax:iconic-focused-color)
  ))
)


;;;;
;;;; Define images
;;;;

;; 1x23
(define title-images
  (list (make-image "title-inactive.png")
        (make-image "title-active.png")))

;; 1x14
(define iconic-title-images
  (lambda () (list
    (make-sized-image 1 14 aquax:iconic-inactive-bg-color)
    (make-sized-image 1 14 aquax:iconic-focused-bg-color))))

;; 6x23
(define title-left-images
  (list (make-image "title-left-inactive.png")
        (make-image "title-left-active.png")))
(define title-right-images
  (list (make-image "title-right-inactive.png")
        (make-image "title-right-active.png")))

(define title-left-shaped-images
  (list (make-image "title-left-inactive-shaped.png")
        (make-image "title-left-active-shaped.png")))
(define title-right-shaped-images
  (list  (make-image "title-right-inactive-shaped.png")
        (make-image "title-right-active-shaped.png")))

;; 19x23
(define close-images `(
  (inactive . ,(make-image "button-inactive.png"))
  (focused . ,(make-image "close-active.png"))
  (highlighted . ,(make-image "close-highlight.png"))
  (inactive-highlighted . ,(make-image "close-inactive-highlight.png"))
  (clicked . ,(make-image "close-clicked.png"))
  (inactive-clicked . ,(make-image "close-inactive-clicked.png"))
))

(define iconify-images `(
  (inactive . ,(make-image "button-inactive.png"))
  (focused . ,(make-image "iconify-active.png"))
  (highlighted . ,(make-image "iconify-highlight.png"))
  (inactive-highlighted . ,(make-image "iconify-inactive-highlight.png"))
  (clicked . ,(make-image "iconify-clicked.png"))
  (inactive-clicked . ,(make-image "iconify-inactive-clicked.png"))
))

(define maximize-images `(
  (inactive . ,(make-image "button-inactive.png"))
  (focused . ,(make-image "maximize-active.png"))
  (highlighted . ,(make-image "maximize-highlight.png"))
  (inactive-highlighted . ,(make-image "maximize-inactive-highlight.png"))
  (clicked . ,(make-image "maximize-clicked.png"))
  (inactive-clicked . ,(make-image "maximize-inactive-clicked.png"))
))

;; menu-images provides a colored menu button whenever window is focused
(define menu-images `(
  (inactive . ,(make-image "button-inactive.png"))
  (focused . ,(make-image "one_window-active.png"))
  (highlighted . ,(make-image "one_window-highlight.png"))
  (inactive-highlighted . ,(make-image "one_window-inactive-highlight.png"))
  (clicked . ,(make-image "one_window-clicked.png"))
  (inactive-clicked . ,(make-image "one_window-inactive-clicked.png"))
))

;; one_window-images provides a look that mimics the Aqua one-window button
(define one_window-images `(
  (inactive . ,(make-image "button-inactive.png"))
  (focused . ,(make-image "one_window-focused-inactive.png"))
  (highlighted . ,(make-image "one_window-highlight.png"))
  (inactive-highlighted . ,(make-image "one_window-inactive-highlight.png"))
  (clicked . ,(make-image "one_window-clicked.png"))
  (inactive-clicked . ,(make-image "one_window-inactive-clicked.png"))
))

;; Shade & sticky images are a special case--we need two sets for each
(define shade-images `(
  (inactive . ,(make-image "button-inactive.png"))
  (focused . ,(make-image "one_window-focused-inactive.png"))
  (highlighted . ,(make-image "shade-highlight.png"))
  (inactive-highlighted . ,(make-image "shade-inactive-highlight.png"))
  (clicked . ,(make-image "shade-clicked.png"))
  (inactive-clicked . ,(make-image "shade-inactive-clicked.png"))
))

(define shade-state-images `(
  (inactive . ,(make-image "shade-inactive.png"))
  (focused . ,(make-image "shade-active.png"))
  (highlighted . ,(make-image "shade-highlight.png"))
  (inactive-highlighted . ,(make-image "shade-inactive-highlight.png"))
  (clicked . ,(make-image "shade-clicked.png"))
  (inactive-clicked . ,(make-image "shade-inactive-clicked.png"))
))

; a 3rd option for those who want the shade button to behave like the others
(define shade-nostate-images `(
  (inactive . ,(make-image "button-inactive.png"))
  (focused . ,(make-image "shade-active.png"))
  (highlighted . ,(make-image "shade-highlight.png"))
  (inactive-highlighted . ,(make-image "shade-inactive-highlight.png"))
  (clicked . ,(make-image "shade-clicked.png"))
  (inactive-clicked . ,(make-image "shade-inactive-clicked.png"))
))

(define sticky-images `(
  (inactive . ,(make-image "button-inactive.png"))
  (focused . ,(make-image "one_window-focused-inactive.png"))
  (highlighted . ,(make-image "sticky-highlight.png"))
  (inactive-highlighted . ,(make-image "sticky-inactive-highlight.png"))
  (clicked . ,(make-image "sticky-clicked.png"))
  (inactive-clicked . ,(make-image "sticky-inactive-clicked.png"))
))

(define sticky-state-images `(
  (inactive . ,(make-image "sticky-inactive.png"))
  (focused . ,(make-image "sticky-active.png"))
  (highlighted . ,(make-image "sticky-highlight.png"))
  (inactive-highlighted . ,(make-image "sticky-inactive-highlight.png"))
  (clicked . ,(make-image "sticky-clicked.png"))
  (inactive-clicked . ,(make-image "sticky-inactive-clicked.png"))
))

;; Define the paired shade/unshade or the alternate shade-nostate image set
(define shade-image-set
  (lambda (w)
    (if aquax:shade-button-state
      (if (or (window-get w 'shaded) (window-get w 'shade-hover-unshaded))
        shade-state-images
        shade-images)
      shade-nostate-images)))

;; Define the paired sticky/unsticky image set
(define sticky-image-set
  (lambda (w)
    (if (window-get w 'sticky)
      sticky-state-images
      sticky-images)))

;; Define the paired one_window/menu image set
(define one_window-image-set
  (lambda ()
    (if aquax:faux-one_window
      one_window-images
      menu-images)))


;; This type of bottom border does not appear in the MacOS X screenshots.
;; My first attempts at the drop-shadow effect failed, so I settled for
;; this kludge--but it's a kludge consistent with the rest of the theme!

;; 1x7
(define bottom-border-images
  (list (make-image "bottom-inactive.png")
        (make-image "bottom-active.png")))

;; 6x7
(define bottom-left-images
  (list (make-image "bottom-left-inactive.png")
        (make-image "bottom-left-active.png")))

(define bottom-right-images
  (list (make-image "bottom-right-inactive.png")
        (make-image "bottom-right-active.png")))

;; 1x1
(define border-pixels
  (list (make-image "border_pixel-inactive.png")
        (make-image "border_pixel-active.png")))


;;;;
;;;; Define the functions that will handle frame variations
;;;;  (especially button arrangements)
;;;;

(define button-offset 5)
(define button-width 19)

(define button-map
  `((iconify-button . ,iconify-images)
    (maximize-button . ,maximize-images)
    (close-button . ,close-images)
    (menu-button . ,one_window-image-set)
    (shade-button . ,shade-image-set)
    (sticky-button . ,sticky-image-set)))

(define (button-theme type)
  (let ((style (cdr (or (assq aquax:button-scheme aquax:button-schemes)
			(assq 'default aquax:button-schemes)))))
    (if (eq type 'transient)
	(cadr style)
      (car style))))

(define (make-buttons spec edge)
  (define (make-button class images point)
    `((background . ,images)
      (,edge . ,point)
      (top-edge . -23)
      (cursor . ,(set-button-cursors class))
      (class . ,class)))

  (do ((rest spec (cdr rest))
       (point button-offset (+ point button-width))
       (out '() (cons (make-button (car rest)
				   (cdr (assq (car rest) button-map))
				   point) out)))
      ((null rest) out)))

(define (title-left-position w)
    (let ((buttons (table-ref button-table w)))
	(+ button-offset (* button-width (length (car buttons))))))
(define (title-right-position w)
    (let ((buttons (table-ref button-table w)))
	(+ button-offset (* button-width (length (cdr buttons))))))


;;;;
;;;; Define frame border groups
;;;;

(define normal-frame `(
  ;; top corners
  ((background . ,title-left-images)
   (top-edge . -23)
   (left-edge . -1)
   (class . top-left-corner))
  ((background . ,title-right-images)
   (top-edge . -23)
   (right-edge . -1)
   (class . top-right-corner))
  ;; titlebar foundation (provides background for removed buttons)
  ((background . ,title-images)
   (top-edge . -23)
   (left-edge . 5)
   (right-edge . 5)
   (cursor . ,(set-button-cursors 'title))
   (class . title))
  ;; actual title space
  ((background . ,title-images)
   (foreground . ,font-colors)
   (text . ,window-name)
   (font . ,aquax:title-font)
   (x-justify . center)
   (y-justify . center)
   (top-edge . -23)
   (left-edge . ,title-left-position)
   (right-edge . ,title-right-position)
   (cursor . ,(set-button-cursors 'title))
   (class . title))
  ;; top border (to allow for border resizing)
  ((background . ,border-pixels)
   (top-edge . -23)
   (left-edge . 5)
   (right-edge . 5)
   (class . top-border))
  ;; bottom border
  ((background . ,bottom-border-images)
   (bottom-edge . -7)
   (right-edge . 5)
   (left-edge . 5)
   (class . bottom-border))
  ((background . ,bottom-left-images)
   (bottom-edge . -7)
   (left-edge . -1)
   (class . bottom-left-corner))
  ((background . ,bottom-right-images)
   (bottom-edge . -7)
   (right-edge . -1)
   (class . bottom-right-corner))
  ;; side borders
  ((background . ,border-pixels)
   (top-edge . 0)
   (bottom-edge . 0)
   (left-edge . -1)
   (class . left-border))
  ((background . ,border-pixels)
   (top-edge . 0)
   (bottom-edge . 0)
   (right-edge . -1)
   (class . right-border))
))
(define shaped-frame `(
  ;; top corners
  ((background . ,title-left-shaped-images)
   (top-edge . -23)
   (left-edge . -1)
   (class . top-left-corner))
  ((background . ,title-right-shaped-images)
   (top-edge . -23)
   (right-edge . -1)
   (class . top-right-corner))
  ;; titlebar foundation (provides background for removed buttons)
  ((background . ,title-images)
   (top-edge . -23)
   (left-edge . 5)
   (right-edge . 5)
   (cursor . ,(set-button-cursors 'title))
   (class . title))
  ;; actual title space
  ((background . ,title-images)
   (foreground . ,font-colors)
   (text . ,window-name)
   (font . ,aquax:title-font)
   (x-justify . center)
   (y-justify . center)
   (top-edge . -23)
   (left-edge . ,title-left-position)
   (right-edge . ,title-right-position)
   (cursor . ,(set-button-cursors 'title))
   (class . title))
  ;; top border (to allow for border resizing)
  ((background . ,border-pixels)
   (top-edge . -23)
   (left-edge . 5)
   (right-edge . 5)
   (class . top-border))
))
(define iconic-title-frame `(
  ((y-justify . center)
   (x-justify . center)
   (text . ,window-name)
   (font . ,aquax:icon-title-font)
   (foreground . ,iconic-font-colors)
   (background . ,iconic-title-images)
   (bottom-edge . -14)
   (right-edge . -10)
   (left-edge . -10)
   (cursor . ,(set-button-cursors 'title))
   (class . iconic-title))
))
; Using the nil-frame didn't work, so we define a null-frame
(define null-frame `(
  ((bottom-edge . 0)
   (right-edge . 0)
   (left-edge . 0))
))


;;;;
;;;; Define functions to handle frame updates correctly
;;;;

(define (rebuild-all)
  (rebuild-frames-with-style theme-name))

(define (reframe-all)
  (reframe-windows-with-style theme-name))

(define (make-frame w frame buttons)
  (table-set button-table w buttons)
  (append frame
	  (make-buttons (car buttons) 'left-edge)
	  (make-buttons (reverse (cdr buttons)) 'right-edge)))

(define (get-frame w type)
  (case type
    ((default)
      (make-frame w normal-frame (button-theme 'normal)))
    ((transient)
      (make-frame w normal-frame (button-theme 'transient)))
    ((shaped)
      (make-frame w shaped-frame (button-theme 'normal)))
    ((shaped-transient)
      (make-frame w shaped-frame (button-theme 'transient)))
    ((icon)
      (if aquax:iconic-titles
        iconic-title-frame
        null-frame))
    ((dock)         null-frame)
    ((unframed)     null-frame)
))


;; register the theme
(add-frame-style theme-name get-frame)

;; update frames when various properties change
(call-after-property-changed 'WM_NAME rebuild-frame)
(call-after-state-changed `(shaded sticky) rebuild-frame)


;; vim: syntax=lisp shiftwidth=2 softtabstop=2

